Chromium Code Reviews
chromiumcodereview-hr@appspot.gserviceaccount.com (chromiumcodereview-hr) | Please choose your nickname with Settings | Help | Chromium Project | Gerrit Changes | Sign out
(204)

Unified Diff: ppapi/examples/audio/audio.cc

Issue 8676043: Pepper: Make the sample rate settable in the audio (output) example plugin. (Closed) Base URL: svn://svn.chromium.org/chrome/trunk/src
Patch Set: Created 9 years, 1 month ago
Use n/p to move between diff chunks; N/P to move between comments. Draft comments are only viewable by you.
Jump to:
View side-by-side diff with in-line comments
Download patch
« no previous file with comments | « no previous file | ppapi/examples/audio/audio.html » ('j') | no next file with comments »
Expand Comments ('e') | Collapse Comments ('c') | Show Comments Hide Comments ('s')
Index: ppapi/examples/audio/audio.cc
diff --git a/ppapi/examples/audio/audio.cc b/ppapi/examples/audio/audio.cc
index 91042fa11fb3fda3d0eb7fa961fbddeecd58102d..589affc54d48eb017139fc3d643117987bc7aa1e 100644
--- a/ppapi/examples/audio/audio.cc
+++ b/ppapi/examples/audio/audio.cc
@@ -2,7 +2,10 @@
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
-#include <cmath>
+#include <math.h>
+#include <stdlib.h>
+#include <string.h>
+
#include <limits>
#include "ppapi/c/pp_errors.h"
@@ -18,57 +21,75 @@ const double frequency_l = 400;
const double frequency_r = 1000;
// This sample frequency is guaranteed to work.
-const PP_AudioSampleRate sample_frequency = PP_AUDIOSAMPLERATE_44100;
-const uint32_t sample_count = 4096;
-uint32_t obtained_sample_count = 0;
+const PP_AudioSampleRate kDefaultSampleRate = PP_AUDIOSAMPLERATE_44100;
+const uint32_t kDefaultSampleCount = 4096;
-const double kPi = 3.141592653589;
-const double kTwoPi = 2.0 * kPi;
+const char kSampleRateAttributeName[] = "samplerate";
class MyInstance : public pp::Instance {
public:
explicit MyInstance(PP_Instance instance)
: pp::Instance(instance),
+ sample_rate_(kDefaultSampleRate),
+ sample_count_(0),
audio_wave_l_(0.0),
audio_wave_r_(0.0) {
}
virtual bool Init(uint32_t argc, const char* argn[], const char* argv[]) {
+ for (uint32_t i = 0; i < argc; i++) {
+ if (strcmp(kSampleRateAttributeName, argn[i]) == 0) {
+ int value = atoi(argv[i]);
+ if (value > 0 && value <= 1000000)
+ sample_rate_ = static_cast<PP_AudioSampleRate>(value);
+ else
+ return false;
+ }
+ }
+
pp::AudioConfig config;
- obtained_sample_count = pp::AudioConfig::RecommendSampleFrameCount(
- sample_frequency, sample_count);
- config = pp::AudioConfig(this, sample_frequency, obtained_sample_count);
- audio_ = pp::Audio(this, config, SineWaveCallback, this);
+ sample_count_ = pp::AudioConfig::RecommendSampleFrameCount(
+ sample_rate_, kDefaultSampleCount);
+ config = pp::AudioConfig(this, sample_rate_, sample_count_);
+ audio_ = pp::Audio(this, config, SineWaveCallbackTrampoline, this);
return audio_.StartPlayback();
}
private:
- static void SineWaveCallback(void* samples, uint32_t num_bytes, void* thiz) {
- const double delta_l = kTwoPi * frequency_l / sample_frequency;
- const double delta_r = kTwoPi * frequency_r / sample_frequency;
+ static void SineWaveCallbackTrampoline(void* samples,
+ uint32_t num_bytes,
+ void* thiz) {
+ static_cast<MyInstance*>(thiz)->SineWaveCallback(samples, num_bytes);
+ }
+
+ void SineWaveCallback(void* samples, uint32_t num_bytes) {
+ double delta_l = 2.0 * M_PI * frequency_l / sample_rate_;
+ double delta_r = 2.0 * M_PI * frequency_r / sample_rate_;
// Use per channel audio wave value to avoid clicks on buffer boundries.
- double wave_l = reinterpret_cast<MyInstance*>(thiz)->audio_wave_l_;
- double wave_r = reinterpret_cast<MyInstance*>(thiz)->audio_wave_r_;
+ double wave_l = audio_wave_l_;
+ double wave_r = audio_wave_r_;
const int16_t max_int16 = std::numeric_limits<int16_t>::max();
int16_t* buf = reinterpret_cast<int16_t*>(samples);
- for (size_t sample = 0; sample < obtained_sample_count; ++sample) {
+ for (size_t sample = 0; sample < sample_count_; ++sample) {
*buf++ = static_cast<int16_t>(sin(wave_l) * max_int16);
*buf++ = static_cast<int16_t>(sin(wave_r) * max_int16);
// Add delta, keep within -kTwoPi..kTwoPi to preserve precision.
wave_l += delta_l;
- if (wave_l > kTwoPi)
- wave_l -= kTwoPi * 2.0;
+ if (wave_l > 2.0 * M_PI)
+ wave_l -= 2.0 * M_PI;
wave_r += delta_r;
- if (wave_r > kTwoPi)
- wave_r -= kTwoPi * 2.0;
+ if (wave_r > 2.0 * M_PI)
+ wave_r -= 2.0 * M_PI;
}
// Store current value to use as starting point for next callback.
- reinterpret_cast<MyInstance*>(thiz)->audio_wave_l_ = wave_l;
- reinterpret_cast<MyInstance*>(thiz)->audio_wave_r_ = wave_r;
+ audio_wave_l_ = wave_l;
+ audio_wave_r_ = wave_r;
}
- // Audio resource. Allocated in Init(), freed on destruction.
+ PP_AudioSampleRate sample_rate_;
+ uint32_t sample_count_;
+
pp::Audio audio_;
// Current audio wave position, used to prevent sine wave skips
« no previous file with comments | « no previous file | ppapi/examples/audio/audio.html » ('j') | no next file with comments »
@@ -18,57 +21,75 @@ const double frequency_l = 400;
*buf++ = static_cast<int16_t>(sin(wave_l) * max_int16);
*buf++ = static_cast<int16_t>(sin(wave_r) * max_int16);
- // Add delta, keep within -kTwoPi..kTwoPi to preserve precision.
+ // Add delta, keep within -2 * M_PI .. 2 * M_PI to preserve precision.
wave_l += delta_l;

Powered by Google App Engine
This is Rietveld 408576698